#########################################################################

### Ecology Letters Paper - Run state-space model code 


#########################################################################

#########################################################################
# DESCRIPTION #
# contains code to run best fitting model (m3.3^251) 1900-2016
# and model m3.3^251 1900-2100 under SSP1 or SSP5 climate scenario 
# with HPAI effect

# code currently set to run model 1900-2016
# lines can be commented in/out as described in model running code and model text file
# to run code under future scenarios
#########################################################################

# open necessary libraries
library(coda)
library(dplyr)
library(tidyr)
library(stringr)
library(rjags)
library(runjags)

#########################################################################

#1. Gannet data preparation

#########################################################################

# set working directory
#setwd("x")

gan1<-read.csv('Ecology_letters_data_run_model/gannets_master_complete_updated.csv')
colonies1<-read.csv('Ecology_letters_data_run_model/Colonies.csv') 

gan1$regional_seas<-as.character(gan1$regional_seas)

gan1<-gan1[-19:-28] # Drops some NA columns
gan1<-gan1[gan1$regional_seas!="Rockall Trough and Bank",]

unique(gan1$regional_seas)

# make new regions based on judgement of distance - EDIT update and rerun model Les Etacs is in wrong region (typo in name)
unique(gan1$Colony)
gan1$regional_seas<-ifelse(gan1$Colony=="Bjornoya","Bjornoya",
                           ifelse(gan1$Colony=="Runde","Southern Norwegian Sea",
                                  ifelse(gan1$Colony=="Kharlov Kola Peninsula","Kharlov Kola Peninsula",
                                         ifelse(gan1$Colony=="Rouzic","South West Atlantic", 
                                                ifelse(gan1$Colony=="Ortac","South West Atlantic",
                                                       ifelse(gan1$Colony=="Les Etacs","South West Atlantic",
                                                              gan1$regional_seas))))))


unique(gan1$regional_seas) # 15 regional seas

gan1$regional_seas<-as.factor(gan1$regional_seas)
#gan1$regional_seas<-droplevels(gan1$regional_seas)
str(gan1)
names(gan1)

gan1$AOS <-as.integer(gan1$AOS)
gan1$colonization_year<-as.character(gan1$colonization_year)
gan1$colonization_year<-as.numeric(gan1$colonization_year)
gan1$extinction_year<-as.character(gan1$extinction_year)
gan1$extinction_year<-as.numeric(gan1$extinction_year)
gan1<-cbind(gan1,region=as.numeric(as.factor(gan1$regional_seas))) # regional seas become numeric regions for matrix below

cols<-unique(gan1$Colony)

## exclude colonisation attempts
## colonisation attempt = 1-2 pairs occupying a location for a few years, typically less than 7, 
## also includes Grimsey and Rockall due to chaotic influence of earthquake/flooding respectively
str(colonies1)
nrow(colonies1)                                                              ## 66 colonies & colonisation events
unique(colonies1[which(colonies1$Status=="colonisation attempt"),4])
length(unique(colonies1[which(colonies1$Status=="colonisation attempt"),4])) ## 13 colonisation events to remove
cols1<-subset(colonies1,colonies1$Status!="colonisation attempt")
cols<-unique(cols1$Colony)

# set number of colonies
N<-length(cols)

# set nYrs to run to present (2016)
nYrs<-117

# UNCOMMENT for future scenarios
# modified nYrs for future scenarios
#nYrs<-201

# UNCOMMENT for future scenarios
# set present year for future scenario
#present<-117

# year to start from for environmental covariates (original year 1900 = 1 => 1950 = 51)
start_Yr<-1

# setup data structures
Pd<-matrix(NA, nrow=N, ncol=nYrs+1) # Matrix of population sizes (colonies in rows, years in columns)
harvest<-Pd*0 # Matrix with information on hunting (yes/no)

reg<-rep(0,N) # Vector of region to which each colony belongs
cc<-rep(0,N) # Vector of terrestrial ccs based on experts' estimates

# sort data for each colony into matrices/vectors
for(n in 1:N){
  # filter for no. years
  # select populations in time frame 
  Pd[n,]<-gan1[gan1$Colony==cols[n],]$AOS[start_Yr:(start_Yr+nYrs)]
  
  # select regions
  reg[n]<-gan1[gan1$Colony==cols[n],]$region[1]             
  
  # select harvest values in time frame
  harvest[n,]<-gan1[gan1$Colony==cols[n],]$harvest[start_Yr:(start_Yr+nYrs)]
  harvest[n,is.na(harvest[n,])]=0
  
  # select terrestrial carrying capacities
  cc[n]<-colonies1[as.character(colonies1$Colony)==as.character(cols[n]),]$Terrestrial.CC
  
  # uncomment to plot population data
  #plot(Pd[n,], main=paste(n,". ",cols[n]), col=harvest[n,]+1, ylim=c(0,cc[n]))
}

# UNCOMMENT for future scenarios
# replace last years of harvest data with last year of data
#harvest[,118:202]<-matrix(harvest[,117],nrow=N, ncol=(202-118+1))

### Read in fecundity data and process
fec_data<-read.csv("Ecology_letters_data_run_model/fecundity_data_sorted.csv")

# filter fec_data for all years up to nYrs-1
fec_data<-fec_data[fec_data$Year<nYrs,]

# UNCOMMENT for future scenarios
# check for fec data for future scenarios
#fec_data<-fec_data[fec_data$Year<present,]

# no. young counted
fledg<-fec_data$totYoung

# no. nests counted
nests<-fec_data$totAON 

# colony ID of counted
nf<-fec_data$ColID

# year of counted
tf<-fec_data$Year

# renumber years according to start Yr
tf<-tf-(start_Yr-1)

fec_obvs<-length(tf)

#########################################################################

#2. Environmental data preparation

#########################################################################

# read in SST data
SST.df<-read.csv("Ecology_letters_data_run_model/weighted_SST_1900_2016_actual.csv")[,2:118]

# convert to matrix form
SST<-as.matrix(SST.df)

# extract mean and sd 
SST_mean<-mean(SST, na.rm=T)
SST_sd<-sd(SST,na.rm=T)

# normalise data
SST<-(SST-mean(SST, na.rm=T))/sd(SST, na.rm=T)

# change all NAs to 0's for entry into model
# mean of data remains centred on 0
SST[is.na(SST)]<-0

# read in nsAT
nsAT.df<-read.csv("Ecology_letters_data_run_model/weighted_nsAT_1900_2016_actual.csv")[,2:118]

# convert to matrix form
nsAT<-as.matrix(nsAT.df)

# calculate means and sds
nsAT_mean<-mean(nsAT, na.rm=T)
nsAT_sd<-sd(nsAT,na.rm=T)

# normalise data
nsAT<-(nsAT-mean(nsAT, na.rm=T))/sd(nsAT, na.rm=T)

# change all NAs to 0's for entry into model
nsAT[is.na(nsAT)]<-0

# read in prec
prec.df<-read.csv("Ecology_letters_data_run_model/weighted_prec_1900_2016_actual.csv")[,2:118]

# convert to matrix form
prec<-as.matrix(prec.df)

prec_mean<-mean(prec, na.rm=T)
prec_sd<-sd(prec,na.rm=T)

# normalise data
prec<-(prec-mean(prec, na.rm=T))/sd(prec, na.rm=T)

# change all NAs to 0's for entry into model
prec[is.na(prec)]<-0

# UNCOMMENT all below to run future scenarios model

# read in future data for all covariates as with historical data above
# normalise covariates with mean and sd from historical data
# SST1.df<-read.csv("Ecology_letters_data_run_model/weighted_SST_SSP1_2015_2100.csv")
# SST5.df<-read.csv("Ecology_letters_data_run_model/weighted_SST_SSP5_2015_2100.csv")
# SST1<-as.matrix(SST1.df)
# SST5<-as.matrix(SST5.df)
# SST1<-(SST1-SST_mean)/SST_sd
# SST5<-(SST5-SST_mean)/SST_sd
# SST1[is.na(SST1)]<-0
# SST5[is.na(SST5)]<-0
# 
# nsAT1.df<-read.csv("Ecology_letters_data_run_model/weighted_nsAT_SSP1_2015_2100.csv")
# nsAT5.df<-read.csv("Ecology_letters_data_run_model/weighted_nsAT_SSP5_2015_2100.csv")
# nsAT1<-as.matrix(nsAT1.df)
# nsAT5<-as.matrix(nsAT5.df)
# nsAT1<-(nsAT1-nsAT_mean)/nsAT_sd
# nsAT5<-(nsAT5-nsAT_mean)/nsAT_sd
# nsAT1[is.na(nsAT1)]<-0
# nsAT5[is.na(nsAT5)]<-0
# 
# prec1.df<-read.csv("Ecology_letters_data_run_model/weighted_prec_SSP1_2015_2100.csv")
# prec5.df<-read.csv("Ecology_letters_data_run_model/weighted_prec_SSP5_2015_2100.csv")
# prec1<-as.matrix(prec1.df)
# prec5<-as.matrix(prec5.df)
# prec1<-(prec1-prec_mean)/prec_sd
# prec5<-(prec5-prec_mean)/prec_sd
# prec1[is.na(prec1)]<-0
# prec5[is.na(prec5)]<-0

### using historical data up-to 2017 (including mean values from 2015 and 2016)

# select future data from 2017 onwards for each relevant covariate
# SST1<-SST1[,3:87]
# SST5<-SST5[,3:87]
# 
# nsAT1<-nsAT1[,3:87]
# nsAT5<-nsAT5[,3:87]
# 
# prec1<-prec1[,3:87]
# prec5<-prec5[,3:87]

### Data to run SSP1 scenario
# bind together historical data with future data 
# SST<-cbind(SST, SST1)
# nsAT<-cbind(nsAT, nsAT1)
# prec<-cbind(prec, prec1)

### Data to run SSP5 scenario
# SST<-cbind(SST, SST5)
# nsAT<-cbind(nsAT, nsAT5)
# prec<-cbind(prec, prec5)

### HPAI - UNCOMMENT TO RUN FUTURE SCENARIO
# setup HPAI matrices to modify survival in 2020 to account for HPAI mortality

# read in matrix of HPAI survivals 
#HPAI<-read.csv("Ecology_letters_data_run_model/HPAI_colonies_matrix.csv")[2:202,]
#HPAI_mat<-as.matrix(HPAI)
# transform matrix to change axes
#HPAI_mat<-t(HPAI_mat)
#HPAI_mat<-HPAI_mat[2:54,]

# change class from character to numeric
#class(HPAI_mat)<-"numeric"

# for any colonies where HPAI occured or might have occured set to 0
#HPAIocc<-HPAI_mat
#HPAIocc[HPAIocc!=1]<-0
#HPAIocc[is.na(HPAIocc)]<-0
# check year 2022
#HPAIocc[,123]

#HPAIs<-HPAI_mat
# for any colonies where HPAI did not occur set to 0
#HPAIs[HPAIs==1]<-0
# check year 2022 
#HPAIs[,123]

#########################################################################

#3. Run model

#########################################################################

# Initial conditions
a0 <- list(chain1=0.00000000001, chain2=0.00000001,chain3=0.000001, chain4=0.000000001)

# set model run specifications
burnin<-15000
n.chains<-4 
sample<-100
thin<-2000
method<-"parallel"

# loop through all model running 
for (i in 1:1){
  
  # run model
  results <- run.jags("Model Text Files/m3.3^251_env_cov_HPAI 0.28.txt", n.chains=n.chains, burnin=burnin, sample=sample, thin=thin, method=method,jags.refresh = 30)
  
  #UNCOMMENT to extend model run
  #results<-extend.jags(results, thin=thin, sample=sample, combine=F)
  
  ## save plots and model output 
  # choose model run number
  run<-"run1"
  model<-"model1"
  
  # generate folder named with model specifics inside working directory
  fn<-paste("Model Runs/plots",n.chains,burnin,sample,thin,method,run,sep="_")
  dir.create(fn)
  
  # save R data for model
  save(file=paste(fn,"/", model, ".Rdata", sep=""), list="results")
  
  # print time taken to run
  print(results$timetaken)
  
  # generature summary statistics
  summaries<-summary(results)
  head(summaries)
  tail(summaries)
  
  # sort summary data frame
  posterior<-data.frame(summaries)
  posterior<-cbind("parameter"=row.names(posterior),posterior)
  posterior$parameter <- gsub("[", ".", posterior$parameter,fixed =TRUE)
  posterior$parameter <- gsub("]", "", posterior$parameter,fixed =TRUE)
  
  head(posterior)
  
  # write summaries to file 
  write.csv(posterior,paste(fn, "/", run, ".estimates.csv", sep=""),row.names=F)
  
  # generate raw mcmc chains
  raw.all <-as.mcmc(results)
  
  raw_all<-data.frame(raw.all)
  
  # save raw mcmc chains to file
  write.csv(raw_all, paste(fn, "/raw_all_", run, ".csv", sep=""), row.names=F)
  
  currentwd<-getwd()
  
  setwd(paste(currentwd, "/",fn,"/", sep=""))
  
  # plot mcmc trace and histogram 
  png()
  
  # plot all model coefficients
  plot(results, plot.type=c("trace","histogram"), vars=c("a0","al0","a2", "aH","aI",
                                                         "sa","sj","K","theta","preceps",
                                                         "SST_a1","nsAT_b","nsAT_b2",
                                                         "prec_b"))
  # turn off photo developing
  dev.off()
  
  setwd(currentwd)
  
  # UNCOMMENT to run DIC fit calculation 
  #for (i in 1:1){
  #  start<-Sys.time()
  # generate DIC
  #  dic.run2H <- extract.runjags(results,"dic", n.iter = 10000, thin = 10, force.resample = FALSE) # Deviance Information Criterion
  #  end<-Sys.time()
  #  start-end
  #}
  
}


